import patches

class PatchSource(object):
    def __iter__(self):
        def iterator(obj):
            for p in obj.read():
                yield p
        return iterator(self)

    def can_live_update(self):
        return False

    def readlines(self):
        raise NotImplementedError()

    def readhunks(self):
        return patches.parse_patches(self.readlines())

class FilePatchSource(PatchSource):
    def __init__(self, filename):
        self.filename = filename
        PatchSource.__init__(self)

    def readlines(self):
        f = open(self.filename, 'r')
        return f.readlines()

class BzrPatchSource(PatchSource):
    def __init__(self, revision=None, file_list=None):
        from bzrlib.branch import Branch
        self.revision = revision
        self.file_list = file_list
        if file_list is not None and len(file_list) > 0:
            location = file_list[0]
        else:
            location = '.'
        self.branch = Branch.open_containing(location)[0]

        PatchSource.__init__(self)

    def can_live_update(self):
        return True

    def readlines(self):
        from StringIO import StringIO
        from bzrlib.diff import show_diff
        f = StringIO()
        show_diff(self.branch, self.revision,
                specific_files=self.file_list, output=f)
        f.seek(0)
        return f.readlines()
